//---------------------------------------------------------------------------


#pragma hdrstop

#include "Settings.h"
#include "KeybKeys.h"
#include "ProgrammableButtons.h"
#include "Branding.h"
#include <algorithm>
#include <fstream>
#include <json/json.h>

//---------------------------------------------------------------------------

#pragma package(smart_init)

Settings appSettings;

inline void strncpyz(char* dst, const char* src, int dstsize) {
	strncpy(dst, src, dstsize);
	dst[dstsize-1] = '\0';
}

void Settings::SetDefault(void)
{
	frmMain.iWidth = 350;
	frmMain.iHeight = 300;
	frmMain.iPosX = 30;
	frmMain.iPosY = 30;
	frmMain.bWindowMaximized = false;
	frmMain.bAlwaysOnTop = false;
	frmMain.bSpeedDialOnly = false;
	frmMain.bXBtnMinimize = false;
	frmMain.bRestoreOnIncomingCall = false;
	frmMain.bSingleInstance = false;
	frmMain.bNoBeepOnEnterKey = false;
	frmMain.bHideSettings = false;

	frmContactPopup.showOnIncoming = false;
	frmContactPopup.showOnOutgoing = false;
	frmContactPopup.iPosX = 100;
	frmContactPopup.iPosY = 100;
	frmContactPopup.iWidth = 400;
	frmContactPopup.iHeight = 140;

	gui.scalingPct = 100;

	Logging.bLogToFile = false;
	Logging.iMaxUiLogLines = 5000;

	Display.bUserOnlyClip = false;

	Integration.asProtocol = Branding::appProto;

	HttpQuery.url = "https://www.google.com/search?q=[number]";
	HttpQuery.openMode = _HttpQuery::openManualOnly;
	Contacts.filterUsingNote = false;

	History.bNoStoreToFile = false;

	Ring.defaultRing = "ring.wav";
	for (int i=0; i<sizeof(Ring.bellcore)/sizeof(Ring.bellcore[0]); i++)
	{
		Ring.bellcore[i] = "ring.wav";
	}

	uaConf.accounts.clear();

	struct UaConf::Account new_acc;
	uaConf.accounts.push_back(new_acc);
}

int Settings::Read(AnsiString asFileName)
{
	Json::Value root;   // will contains the root value after parsing.
	Json::Reader reader;

    SetDefault();

	try
	{
		std::ifstream ifs(asFileName.c_str());
		std::string strConfig((std::istreambuf_iterator<char>(ifs)), std::istreambuf_iterator<char>());
		ifs.close();
		bool parsingSuccessful = reader.parse( strConfig, root );
		if ( !parsingSuccessful )
		{
			return 2;
		}
	}
	catch(...)
	{
		return 1;
	}

	const Json::Value &uaConfJson = root["uaConf"];
	const Json::Value &uaConfAudioCfgSrcJson = uaConfJson["audioCfgSrc"];
	strncpyz(uaConf.audioCfgSrc.mod, uaConfAudioCfgSrcJson.get("mod", UaConf::modWinwave).asString().c_str(), sizeof(uaConf.audioCfgSrc.mod));
	strncpyz(uaConf.audioCfgSrc.dev, uaConfAudioCfgSrcJson["dev"].asString().c_str(), sizeof(uaConf.audioCfgSrc.dev));
	strncpyz(uaConf.audioCfgSrc.wavefile, uaConfAudioCfgSrcJson["wavefile"].asString().c_str(), sizeof(uaConf.audioCfgSrc.wavefile));
	if (strcmp(uaConf.audioCfgSrc.mod, UaConf::modPortaudio) &&
		strcmp(uaConf.audioCfgSrc.mod, UaConf::modWinwave) &&
		strcmp(uaConf.audioCfgSrc.mod, UaConf::modAufile)) {
		strncpyz(uaConf.audioCfgSrc.mod, UaConf::modWinwave, sizeof(uaConf.audioCfgSrc.mod));
	}

	const Json::Value &uaConfAudioCfgPlayJson = uaConfJson["audioCfgPlay"];
	strncpyz(uaConf.audioCfgPlay.mod, uaConfAudioCfgPlayJson.get("mod", UaConf::modWinwave).asString().c_str(), sizeof(uaConf.audioCfgPlay.mod));
	strncpyz(uaConf.audioCfgPlay.dev, uaConfAudioCfgPlayJson["dev"].asString().c_str(), sizeof(uaConf.audioCfgPlay.dev));
	if (strcmp(uaConf.audioCfgPlay.mod, UaConf::modPortaudio) &&
		strcmp(uaConf.audioCfgPlay.mod, UaConf::modWinwave)) {
		strncpyz(uaConf.audioCfgPlay.mod, UaConf::modWinwave, sizeof(uaConf.audioCfgPlay.mod));
	}
	const Json::Value &uaConfAudioCfgAlertJson = uaConfJson["audioCfgAlert"];
	strncpyz(uaConf.audioCfgAlert.mod, uaConfAudioCfgAlertJson.get("mod", UaConf::modWinwave).asString().c_str(), sizeof(uaConf.audioCfgAlert.mod));
	strncpyz(uaConf.audioCfgAlert.dev, uaConfAudioCfgAlertJson["dev"].asString().c_str(), sizeof(uaConf.audioCfgAlert.dev));
	if (strcmp(uaConf.audioCfgAlert.mod, UaConf::modPortaudio) &&
		strcmp(uaConf.audioCfgAlert.mod, UaConf::modWinwave)) {
		strncpyz(uaConf.audioCfgAlert.mod, UaConf::modWinwave, sizeof(uaConf.audioCfgAlert.mod));
	}
	const Json::Value &uaConfAudioCfgPlayIntercomJson = uaConfJson["audioCfgPlayIntercom"];
	strncpyz(uaConf.audioCfgPlayIntercom.mod, uaConfAudioCfgPlayIntercomJson.get("mod", UaConf::modWinwave).asString().c_str(), sizeof(uaConf.audioCfgPlayIntercom.mod));
	strncpyz(uaConf.audioCfgPlayIntercom.dev, uaConfAudioCfgPlayIntercomJson["dev"].asString().c_str(), sizeof(uaConf.audioCfgPlayIntercom.dev));
	if (strcmp(uaConf.audioCfgPlayIntercom.mod, UaConf::modPortaudio) &&
		strcmp(uaConf.audioCfgPlayIntercom.mod, UaConf::modWinwave)) {
		strncpyz(uaConf.audioCfgPlayIntercom.mod, UaConf::modWinwave, sizeof(uaConf.audioCfgPlayIntercom.mod));
	}

	if (Branding::recording)
	{
		const Json::Value &uaConfRecordingJson = uaConfJson["recording"];
		uaConf.recording.enabled = uaConfRecordingJson.get("enabled", false).asBool();
		uaConf.recording.recDir = static_cast<UaConf::RecordingCfg::RecDir>(uaConfRecordingJson.get("recDir", UaConf::RecordingCfg::RecDirRelative).asInt());
		if (uaConf.recording.recDir < 0 || uaConf.recording.recDir >= UaConf::RecordingCfg::RecDirLimiter) {
			uaConf.recording.recDir = UaConf::RecordingCfg::RecDirRelative;
		}
		uaConf.recording.customRecDir = uaConfRecordingJson.get("customRecDir", "").asString();
		uaConf.recording.channels = uaConfRecordingJson.get("channels", uaConf.recording.channels).asUInt();
		if (uaConf.recording.channels < UaConf::RecordingCfg::CHANNELS_MIN || uaConf.recording.channels > UaConf::RecordingCfg::CHANNELS_MAX) {
			uaConf.recording.channels = UaConf::RecordingCfg::CHANNELS_MIN;
		}
	}
	else
	{
		uaConf.recording.enabled = false;
	}

	uaConf.aec = (enum UaConf::Aec)uaConfJson.get("aec", UaConf::AEC_WEBRTC).asInt();
	if (uaConf.aec < 0 || uaConf.aec >= UaConf::AEC_LIMIT)
	{
    	uaConf.aec = UaConf::AEC_WEBRTC;
	}

	const Json::Value &webrtcAec = root["webrtcAec"];
	uaConf.webrtcAec.msInSndCardBuf = webrtcAec.get("msInSndCardBuf", 120).asInt();
	uaConf.webrtcAec.skew = webrtcAec.get("skew", 0).asInt();

	uaConf.logMessages = uaConfJson.get("logMessages", false).asBool();
	uaConf.local = uaConfJson.get("localAddress", "").asString();
	uaConf.ifname = uaConfJson.get("ifName", "").asString();

	const Json::Value &uaAvtJson = uaConfJson["avt"];
	uaConf.avt.portMin = uaAvtJson.get("portMin", uaConf.avt.portMin).asUInt();
	uaConf.avt.portMax = uaAvtJson.get("portMax", uaConf.avt.portMax).asUInt();
	uaConf.avt.jbufDelayMin = uaAvtJson.get("jbufDelayMin", uaConf.avt.jbufDelayMin).asUInt();
	uaConf.avt.jbufDelayMax = uaAvtJson.get("jbufDelayMax", uaConf.avt.jbufDelayMax).asUInt();
	uaConf.avt.Validate();	

	uaConf.autoAnswer = uaConfJson.get("autoAnswer", false).asBool();
	uaConf.autoAnswerCode = uaConfJson.get("autoAnswerCode", 200).asInt();
	uaConf.autoAnswerDelayMin = uaConfJson.get("autoAnswerDelayMin", 0). asUInt();
	uaConf.autoAnswerDelayMax = uaConfJson.get("autoAnswerDelayMax", 0). asUInt();
	if (uaConf.autoAnswerDelayMin > uaConf.autoAnswerDelayMax)
	{
		uaConf.autoAnswerDelayMin = 0;
		uaConf.autoAnswerDelayMax = 0;
	}
	uaConf.autoAnswerCallInfo = uaConfJson.get("autoAnswerCallInfo", false).asBool();
	uaConf.autoAnswerCallInfoDelayMin = uaConfJson.get("autoAnswerCallInfoDelayMin", 0).asUInt();

	uaConf.answerOnEventTalk = uaConfJson.get("answerOnEventTalk", false).asBool();

	const Json::Value &accounts = root["Accounts"];
	//uaConf.accounts.clear();
	for (int i=0; i<std::min(1u, accounts.size()); i++)
	{
		const Json::Value &acc = accounts[i];
		struct UaConf::Account new_acc;

		new_acc.reg_server = acc.get("reg_server", "").asString();
		new_acc.user = acc.get("user", "").asString();
		new_acc.auth_user = acc.get("auth_user", "").asString();
		new_acc.pwd = acc.get("pwd", "").asString();
		new_acc.transport =
			(UaConf::Account::TRANSPORT_TYPE)acc.get("transport", UaConf::Account::TRANSPORT_UDP).asInt();
		if (new_acc.transport < 0 || new_acc.transport >= UaConf::Account::TRANSPORT_LIMITER)
		{
			new_acc.transport = UaConf::Account::TRANSPORT_UDP;
		}
		// do not register by default
		// (long timeout when unregistering from non-existing server, when application is closing
		// or when account settings are changed to valid)
		new_acc.reg_expires = acc.get("reg_expires", 0).asInt();
		if (new_acc.reg_expires < 0 || new_acc.reg_expires > 7200)
		{
			new_acc.reg_expires = 60;
		}
		new_acc.answer_any = acc.get("answer_any", false).asBool();

		new_acc.stun_server = acc.get("stun_server", "").asString();
		new_acc.outbound1 = acc.get("outbound1", "").asString();
		new_acc.outbound2 = acc.get("outbound2", "").asString();

		const Json::Value &audio_codecs = acc["audio_codecs"];
		if (audio_codecs.type() == Json::arrayValue)
		{
            new_acc.audio_codecs.clear();
			for (int i=0; i<audio_codecs.size(); i++)
			{
				const Json::Value &codec = audio_codecs[i];
				new_acc.audio_codecs.push_back(codec.asString());
			}
		}
		else
		{
			// default
			new_acc.audio_codecs.push_back("PCMU/8000/1");
			new_acc.audio_codecs.push_back("PCMA/8000/1");
		}

		uaConf.accounts[0] = new_acc;
	}
#if 0	// obsolete, replaced with "btnConf"
	const Json::Value &contacts = root["Contacts"];
	uaConf.contacts.clear();
	for (int i=0; i<contacts.size(); i++)
	{
		const Json::Value &contact = contacts[i];
		struct UaConf::Contact new_contact;

		new_contact.description = contact.get("description", "").asString();
		new_contact.user = contact.get("user", "").asString();
		new_contact.sub_dialog_info = contact.get("sub_dialog_info", false).asBool();
		new_contact.sub_presence = contact.get("sub_presence", false).asBool();
		uaConf.contacts.push_back(new_contact);
	}

	for (int i=contacts.size(); i<UaConf::CONTACTS_CNT; i++)
	{
		struct UaConf::Contact new_contact;
		uaConf.contacts.push_back(new_contact);
	}
#endif

	const Json::Value &hotkeyConfJson = root["hotkeyConf"];
	for (int i=0; i<hotkeyConfJson.size(); i++)
	{
		const Json::Value &hotkeyJson = hotkeyConfJson[i];
		class HotKeyConf cfg;
		cfg.keyCode = hotkeyJson.get("keyCode", "").asString().c_str();
		int id = vkey_find(cfg.keyCode.c_str());
		if (id >= 0)
		{
        	cfg.vkCode = vkey_list[id].vkey;
		}
		else
		{
        	cfg.vkCode = -1;
		}
		cfg.modifiers = hotkeyJson.get("modifiers", 0).asInt();
		cfg.global = hotkeyJson.get("global", false).asBool();
		const Json::Value &action = hotkeyJson["action"];
		cfg.action.type = static_cast<Action::Type>(action.get("type", 0).asInt());
		cfg.action.id = action.get("id", 0).asInt();

		hotKeyConf.push_back(cfg); 
	}

	const Json::Value &phoneConfJson = root["phoneConf"];
	for (int i=0; i<phoneConfJson.size(); i++)
	{
		const Json::Value &phoneJson = phoneConfJson[i];
		PhoneConf cfg;
		cfg.dllName = phoneJson.get("dllName", "").asString().c_str();
		if (cfg.dllName != "")
		{
        	phoneConf.push_back(cfg);
		}
	}

	const Json::Value &guiJson = root["gui"];
	gui.scalingPct = guiJson.get("scalingPct", 100).asInt();
	if (gui.scalingPct < gui.SCALING_MIN || gui.scalingPct > gui.SCALING_MAX) {
    	gui.scalingPct = 100;    
	}

	int maxX = GetSystemMetrics(SM_CXSCREEN);
	/** \todo Ugly fixed taskbar margin */
	int maxY = GetSystemMetrics(SM_CYSCREEN) - 32;

	const Json::Value &frmMainJson = root["frmMain"];
	frmMain.iWidth = frmMainJson.get("AppWidth", 350).asInt();
	frmMain.iHeight = frmMainJson.get("AppHeight", 300).asInt();
	if (frmMain.iWidth < 250 || frmMain.iWidth > maxX + 20)
	{
		frmMain.iWidth = 250;
	}
	if (frmMain.iHeight < 200 || frmMain.iHeight > maxY + 20)
	{
		frmMain.iHeight = 200;
	}
	frmMain.iPosX = frmMainJson.get("AppPositionX", 30).asInt();
	frmMain.iPosY = frmMainJson.get("AppPositionY", 30).asInt();
	if (frmMain.iPosX < 0)
		frmMain.iPosX = 0;
	if (frmMain.iPosY < 0)
		frmMain.iPosY = 0;
	if (frmMain.iPosX + frmMain.iWidth > maxX)
		frmMain.iPosX = maxX - frmMain.iWidth;
	if (frmMain.iPosY + frmMain.iHeight > maxY)
		frmMain.iPosY = maxY - frmMain.iHeight;
	frmMain.bWindowMaximized = frmMainJson.get("Maximized", false).asBool();
	frmMain.bAlwaysOnTop = frmMainJson.get("AlwaysOnTop", false).asBool();
	frmMain.bSpeedDialVisible = frmMainJson.get("SpeedDialVisible", false).asBool();
	frmMain.bSpeedDialOnly = frmMainJson.get("SpeedDialOnly", false).asBool();
	frmMain.iSpeedDialSize = frmMainJson.get("SpeedDialSize", 1).asUInt();
	if (frmMain.iSpeedDialSize > ProgrammableButtons::EXT_CONSOLE_COLUMNS)
		frmMain.iSpeedDialSize = 0;
	frmMain.iSpeedDialWidth = frmMainJson.get("SpeedDialWidth", 105).asUInt();
	if (frmMain.iSpeedDialWidth < 50 || frmMain.iSpeedDialWidth > 300)
        frmMain.iSpeedDialWidth = 105;
	frmMain.bStartMinimizedToTray = frmMainJson.get("StartMinimizedToTray", false).asBool();
	frmMain.bXBtnMinimize = frmMainJson.get("XBtnMinimize", false).asBool();
	frmMain.bRestoreOnIncomingCall = frmMainJson.get("RestoreOnIncomingCall", false).asBool();
	frmMain.bSingleInstance = frmMainJson.get("SingleInstance", frmMain.bSingleInstance).asBool();
	frmMain.dialpadBackgroundImage = frmMainJson.get("DialpadBackgroundImage", frmMain.dialpadBackgroundImage.c_str()).asString().c_str();
	frmMain.mainIcon = frmMainJson.get("MainIcon", frmMain.mainIcon.c_str()).asString().c_str();
	frmMain.trayNotificationImage = frmMainJson.get("TrayNotificationImage", frmMain.trayNotificationImage.c_str()).asString().c_str();

	frmMain.bNoBeepOnEnterKey = frmMainJson.get("NoBeepOnEnterKey", frmMain.bNoBeepOnEnterKey).asBool();
	frmMain.bHideSettings = frmMainJson.get("HideSettings", frmMain.bHideSettings).asBool();

	const Json::Value &frmTrayNotifierJson = root["frmTrayNotifier"];
	frmTrayNotifier.iHeight = 105;
	frmTrayNotifier.iWidth = 213;
	frmTrayNotifier.iPosX = frmTrayNotifierJson.get("PosX", maxX - frmTrayNotifier.iWidth).asInt();
	frmTrayNotifier.iPosY = frmTrayNotifierJson.get("PosY", maxY - frmTrayNotifier.iHeight).asInt();
	if (frmTrayNotifier.iPosX < 0)
		frmTrayNotifier.iPosX = 0;
	if (frmTrayNotifier.iPosY < 0)
		frmTrayNotifier.iPosY = 0;
	if (frmTrayNotifier.iPosX + frmMain.iWidth > maxX)
		frmTrayNotifier.iPosX = maxX - frmTrayNotifier.iWidth;
	if (frmTrayNotifier.iPosY + frmMain.iHeight > maxY)
		frmTrayNotifier.iPosY = maxY - frmTrayNotifier.iHeight;
	frmTrayNotifier.showOnIncoming = frmTrayNotifierJson.get("ShowOnIncoming", false).asBool();
	frmTrayNotifier.skipIfMainWindowVisible = frmTrayNotifierJson.get("SkipIfMainWindowVisible", false).asBool();
	frmTrayNotifier.showOnOutgoing = frmTrayNotifierJson.get("ShowOnOutgoing", false).asBool();
	frmTrayNotifier.backgroundImage = frmTrayNotifierJson.get("BackgroundImage", frmTrayNotifier.backgroundImage.c_str()).asString().c_str();

	const Json::Value &frmContactPopupJson = root["frmContactPopup"];
	frmContactPopup.showOnIncoming = frmContactPopupJson.get("ShowOnIncoming", false).asBool();
	frmContactPopup.showOnOutgoing = frmContactPopupJson.get("ShowOnOutgoing", false).asBool();
	frmContactPopup.iPosX = frmContactPopupJson.get("PosX", 100).asInt();
	frmContactPopup.iPosY = frmContactPopupJson.get("PosY", 100).asInt();
	frmContactPopup.iWidth = frmContactPopupJson.get("Width", 400).asInt();
	frmContactPopup.iHeight = frmContactPopupJson.get("Height", 140).asInt();

	const Json::Value &LoggingJson = root["Logging"];
	Logging.bLogToFile = LoggingJson.get("LogToFile", false).asBool();
	Logging.iMaxUiLogLines = LoggingJson.get("MaxUiLogLines", 5000).asInt();

	const Json::Value &DisplayJson = root["Display"];
	Display.bUserOnlyClip = DisplayJson.get("UserOnlyClip", Display.bUserOnlyClip).asBool();

	const Json::Value &IntegrationJson = root["Integration"];
	Integration.bAddFilterWMCopyData = IntegrationJson.get("AddFilterWMCopyData", false).asBool();
	Integration.asProtocol = IntegrationJson.get("Protocol", Branding::appProto.c_str()).asString().c_str();

	const Json::Value &RingJson = root["Ring"];
	Ring.defaultRing = RingJson.get("DefaultRing", "ring.wav").asString().c_str();
	const Json::Value &bellcore = RingJson["Bellcore"];
	for (int i=0; i<std::min(sizeof(Ring.bellcore)/sizeof(Ring.bellcore[0]), bellcore.size()); i++)
	{
		Ring.bellcore[i] = bellcore[i].asString().c_str();
	}

	const Json::Value &HttpQueryJson = root["HttpQuery"];
	HttpQuery.url = HttpQueryJson.get("Url", HttpQuery.url.c_str()).asString().c_str();
	HttpQuery.openMode = static_cast<_HttpQuery::OpenMode>(HttpQueryJson.get("OpenMode", HttpQuery.openMode).asInt());
	if (HttpQuery.openMode < 0 || HttpQuery.openMode >= _HttpQuery::openModeLimiter)
	{
        HttpQuery.openMode = _HttpQuery::openManualOnly;
    }

	const Json::Value &ContactsJson = root["Contacts"];
	Contacts.filterUsingNote = ContactsJson.get("FilterUsingNote", Contacts.filterUsingNote).asBool();

	const Json::Value &HistoryJson = root["History"];
	History.bNoStoreToFile = HistoryJson.get("NoStoreToFile", History.bNoStoreToFile).asBool();

	return 0;
}

int Settings::Write(AnsiString asFileName)
{
    Json::Value root;
	Json::StyledWriter writer;

	root["gui"]["scalingPct"] = gui.scalingPct;

	root["frmMain"]["AppWidth"] = frmMain.iWidth;
	root["frmMain"]["AppHeight"] = frmMain.iHeight;
	root["frmMain"]["AppPositionX"] = frmMain.iPosX;
	root["frmMain"]["AppPositionY"] = frmMain.iPosY;
	root["frmMain"]["Maximized"] = frmMain.bWindowMaximized;
	root["frmMain"]["AlwaysOnTop"] = frmMain.bAlwaysOnTop;
	root["frmMain"]["SpeedDialVisible"] = frmMain.bSpeedDialVisible;
	root["frmMain"]["SpeedDialOnly"] = frmMain.bSpeedDialOnly;
	root["frmMain"]["SpeedDialSize"] = frmMain.iSpeedDialSize;
	root["frmMain"]["SpeedDialWidth"] = frmMain.iSpeedDialWidth;
	root["frmMain"]["StartMinimizedToTray"] = frmMain.bStartMinimizedToTray;
	root["frmMain"]["XBtnMinimize"] = frmMain.bXBtnMinimize;
	root["frmMain"]["RestoreOnIncomingCall"] = frmMain.bRestoreOnIncomingCall;
	root["frmMain"]["SingleInstance"] = frmMain.bSingleInstance;
	root["frmMain"]["DialpadBackgroundImage"] = frmMain.dialpadBackgroundImage.c_str();
	root["frmMain"]["MainIcon"] = frmMain.mainIcon.c_str();
	root["frmMain"]["TrayNotificationImage"] = frmMain.trayNotificationImage.c_str();
	root["frmMain"]["NoBeepOnEnterKey"] = frmMain.bNoBeepOnEnterKey;
	root["frmMain"]["HideSettings"] = frmMain.bHideSettings;

	root["frmTrayNotifier"]["PosX"] = frmTrayNotifier.iPosX;
	root["frmTrayNotifier"]["PosY"] = frmTrayNotifier.iPosY;
	root["frmTrayNotifier"]["ShowOnIncoming"] = frmTrayNotifier.showOnIncoming;
	root["frmTrayNotifier"]["SkipIfMainWindowVisible"] = frmTrayNotifier.skipIfMainWindowVisible;
	root["frmTrayNotifier"]["ShowOnOutgoing"] = frmTrayNotifier.showOnOutgoing;
	root["frmTrayNotifier"]["BackgroundImage"] = frmTrayNotifier.backgroundImage.c_str();

	root["frmContactPopup"]["ShowOnIncoming"] = frmContactPopup.showOnIncoming;
	root["frmContactPopup"]["ShowOnOutgoing"] = frmContactPopup.showOnOutgoing;
	root["frmContactPopup"]["PosX"] = frmContactPopup.iPosX;
	root["frmContactPopup"]["PosY"] = frmContactPopup.iPosY;
	root["frmContactPopup"]["Width"] = frmContactPopup.iWidth;
	root["frmContactPopup"]["Height"] = frmContactPopup.iHeight;

	root["Logging"]["LogToFile"] = Logging.bLogToFile;
	root["Logging"]["MaxUiLogLines"] = Logging.iMaxUiLogLines;

	root["Display"]["UserOnlyClip"] = Display.bUserOnlyClip;

	root["Integration"]["AddFilterWMCopyData"] = Integration.bAddFilterWMCopyData;
	root["Integration"]["Protocol"] = Integration.asProtocol.c_str();

	root["Ring"]["DefaultRing"] = Ring.defaultRing.c_str();
	for (int i=0; i<sizeof(Ring.bellcore)/sizeof(Ring.bellcore[0]); i++)
	{
		root["Ring"]["Bellcore"][i] = Ring.bellcore[i].c_str();
	}

	root["HttpQuery"]["Url"] = HttpQuery.url.c_str();
	root["HttpQuery"]["OpenMode"] = HttpQuery.openMode;

	root["Contacts"]["FilterUsingNote"] = Contacts.filterUsingNote;

	root["History"]["NoStoreToFile"] = History.bNoStoreToFile;

	root["uaConf"]["audioCfgSrc"]["mod"] = uaConf.audioCfgSrc.mod;
	root["uaConf"]["audioCfgSrc"]["dev"] = uaConf.audioCfgSrc.dev;
	root["uaConf"]["audioCfgSrc"]["wavefile"] = uaConf.audioCfgSrc.wavefile;
	root["uaConf"]["audioCfgPlay"]["mod"] = uaConf.audioCfgPlay.mod;
	root["uaConf"]["audioCfgPlay"]["dev"] = uaConf.audioCfgPlay.dev;
	root["uaConf"]["audioCfgAlert"]["mod"] = uaConf.audioCfgAlert.mod;
	root["uaConf"]["audioCfgAlert"]["dev"] = uaConf.audioCfgAlert.dev;
	root["uaConf"]["audioCfgPlayIntercom"]["mod"] = uaConf.audioCfgPlayIntercom.mod;
	root["uaConf"]["audioCfgPlayIntercom"]["dev"] = uaConf.audioCfgPlayIntercom.dev;

	if (Branding::recording)
	{
		root["uaConf"]["recording"]["enabled"] = uaConf.recording.enabled;
		root["uaConf"]["recording"]["recDir"] = uaConf.recording.recDir;
		root["uaConf"]["recording"]["customRecDir"] = uaConf.recording.customRecDir;
		root["uaConf"]["recording"]["channels"] = uaConf.recording.channels;
	}

	root["uaConf"]["aec"] = uaConf.aec;
	root["uaConf"]["logMessages"] = uaConf.logMessages;
	
	root["uaConf"]["localAddress"] = uaConf.local;
	root["uaConf"]["ifName"] = uaConf.ifname;
	root["uaConf"]["avt"]["portMin"] = uaConf.avt.portMin;
	root["uaConf"]["avt"]["portMax"] = uaConf.avt.portMax;
	root["uaConf"]["avt"]["jbufDelayMin"] = uaConf.avt.jbufDelayMin;
	root["uaConf"]["avt"]["jbufDelayMax"] = uaConf.avt.jbufDelayMax;

	root["uaConf"]["autoAnswer"] = uaConf.autoAnswer;
	root["uaConf"]["autoAnswerCode"] = uaConf.autoAnswerCode;
	root["uaConf"]["autoAnswerDelayMin"] = uaConf.autoAnswerDelayMin;
	root["uaConf"]["autoAnswerDelayMax"] = uaConf.autoAnswerDelayMax;
	root["uaConf"]["autoAnswerCallInfo"] = uaConf.autoAnswerCallInfo;
	root["uaConf"]["autoAnswerCallInfoDelayMin"] = uaConf.autoAnswerCallInfoDelayMin;

    root["uaConf"]["answerOnEventTalk"] = uaConf.answerOnEventTalk;

	root["uaConf"]["webrtcAec"]["msInSndCardBuf"] = uaConf.webrtcAec.msInSndCardBuf;
	root["uaConf"]["webrtcAec"]["skew"] = uaConf.webrtcAec.skew;

	// write accounts
	for (unsigned int i=0; i<uaConf.accounts.size(); i++)
	{
		UaConf::Account &acc = uaConf.accounts[i];
		root["Accounts"][i]["reg_server"] = acc.reg_server;
		root["Accounts"][i]["user"] = acc.user;
		root["Accounts"][i]["auth_user"] = acc.auth_user;
		root["Accounts"][i]["pwd"] = acc.pwd;
		root["Accounts"][i]["transport"] = acc.transport;
		root["Accounts"][i]["reg_expires"] = acc.reg_expires;
		root["Accounts"][i]["answer_any"] = acc.answer_any;
		root["Accounts"][i]["stun_server"] = acc.stun_server;
		root["Accounts"][i]["outbound1"] = acc.outbound1;
		root["Accounts"][i]["outbound2"] = acc.outbound2;
		for (unsigned int j=0; j<acc.audio_codecs.size(); j++)
		{
			root["Accounts"][i]["audio_codecs"][j] = acc.audio_codecs[j];
		}
	}
#if 0	// obsolete, replaced with "btnConf"
	// write contacts
	for (unsigned int i=0; i<uaConf.contacts.size(); i++)
	{
		UaConf::Contact &contact = uaConf.contacts[i];
		root["Contacts"][i]["description"] = contact.description;
		root["Contacts"][i]["user"] = contact.user;
		root["Contacts"][i]["sub_dialog_info"] = contact.sub_dialog_info;
		root["Contacts"][i]["sub_presence"] = contact.sub_presence;
	}
#endif

	{
		int i = 0;
		std::list<HotKeyConf>::iterator iter;
		for (iter = hotKeyConf.begin(); iter != hotKeyConf.end(); ++iter)
		{
			struct HotKeyConf &cfg = *iter;
			Json::Value &cfgJson = root["hotkeyConf"][i++];
			cfgJson["keyCode"] = cfg.keyCode.c_str();
			cfgJson["modifiers"] = cfg.modifiers;
			cfgJson["global"] = cfg.global;
			cfgJson["action"]["type"] = cfg.action.type;
			cfgJson["action"]["id"] = cfg.action.id;
		}
	}

	{
		int i = 0;
		std::list<PhoneConf>::iterator iter;
		for (iter = phoneConf.begin(); iter != phoneConf.end(); ++iter)
		{
			PhoneConf &cfg = *iter;
			Json::Value &cfgJson = root["phoneConf"][i++];
			cfgJson["dllName"] = cfg.dllName.c_str();
		}
	}

	std::string outputConfig = writer.write( root );

	try
	{
		std::ofstream ofs(asFileName.c_str());
		ofs << outputConfig;
		ofs.close();
	}
	catch(...)
	{
    	return 1;
	}

	return 0;
}


